import tkinter
from tkinter import ttk
from magictk import color_tmpl
from magictk import photoload


def exit_btn_set(self, root):
    img_exit = photoload.loadimg("close")
    img_act_exit = photoload.loadimg("close_active")
    self.style.configure(
        'BarExitButton.TFrame', background=self.color["background"])
    self.style.configure(
        'BarExitButton.TLabel', background=self.color["background"])
    exit_btn = ttk.Frame(root, style='BarExitButton.TFrame',
                         height=self.top_h, width=48)
    exit_btn.pack(anchor="n", side='right')
    exit_btn_flash = 0  # total 6f
    exit_btn_flash_mode = 0
    label = ttk.Label(
        exit_btn, style='BarExitButton.TLabel', image=img_exit, anchor=tkinter.CENTER)
    label.place(width=48, height=self.top_h, x=0, y=0)

    def animation():
        nonlocal exit_btn_flash, exit_btn_flash_mode
        if (exit_btn_flash_mode == 0 and exit_btn_flash > 0):
            exit_btn_flash -= 1
            self.style.configure(
                'BarExitButton.TFrame', background=color_tmpl.mix_color(self.color["background"], "#C42B1C", exit_btn_flash/6))
            self.style.configure(
                'BarExitButton.TLabel', background=color_tmpl.mix_color(self.color["background"], "#C42B1C", exit_btn_flash/6))
            if (exit_btn_flash >= 3):
                label.configure(image=img_act_exit)
            else:
                label.configure(image=img_exit)
        if (exit_btn_flash_mode == 1 and exit_btn_flash < 6):
            exit_btn_flash += 1
            self.style.configure(
                'BarExitButton.TFrame', background=color_tmpl.mix_color(self.color["background"], "#C42B1C", exit_btn_flash/6))
            self.style.configure(
                'BarExitButton.TLabel', background=color_tmpl.mix_color(self.color["background"], "#C42B1C", exit_btn_flash/6))
            if (exit_btn_flash >= 3):
                label.configure(image=img_act_exit)
            else:
                label.configure(image=img_exit)
        if (exit_btn_flash_mode == 2):
            exit_btn_flash = 6
            self.style.configure(
                'BarExitButton.TFrame', background="#94141E")
            self.style.configure(
                'BarExitButton.TLabel', background="#94141E")
            if (exit_btn_flash >= 3):
                label.configure(image=img_act_exit)
            else:
                label.configure(image=img_exit)
    self.anim.append(animation)

    def enters_exit(event):
        nonlocal exit_btn_flash_mode
        exit_btn_flash_mode = 1

    def leaves_exit(event):
        nonlocal exit_btn_flash_mode
        exit_btn_flash_mode = 0

    def presshold_exit(event):
        nonlocal exit_btn_flash_mode
        exit_btn_flash_mode = 2

    def press_exit(event):
        nonlocal exit_btn_flash_mode
        if (exit_btn_flash_mode == 2):
            exit_btn_flash_mode = 0
            self.quit()
        else:
            exit_btn_flash_mode = 0
    exit_btn.bind("<Enter>", enters_exit)
    exit_btn.bind("<Leave>", leaves_exit)
    exit_btn.bind("<Button-1>", presshold_exit)
    exit_btn.bind("<ButtonRelease-1>", press_exit)
    label.bind("<Enter>", enters_exit)
    label.bind("<Leave>", leaves_exit)
    label.bind("<Button-1>", presshold_exit)
    label.bind("<ButtonRelease-1>", press_exit)


def zoom_btn_set(self, root):
    img_default = photoload.loadimg("zoom")
    img_inzoom = photoload.loadimg("zoom2normal")

    self.style.configure(
        'BarZoomButton.TFrame', background=self.color["background"])
    self.style.configure(
        'BarZoomButton.TLabel', background=self.color["background"])
    zoom_btn = ttk.Frame(root, style='BarZoomButton.TFrame',
                         height=self.top_h, width=48)
    zoom_btn.pack(anchor="n", side='right')
    zoom_btn_flash = 0  # total 4f
    zoom_btn_flash_mode = 0
    label = ttk.Label(
        zoom_btn, style='BarZoomButton.TLabel', image=img_default, anchor=tkinter.CENTER)
    label.place(width=48, height=self.top_h, x=0, y=0)

    def animation():
        nonlocal zoom_btn_flash, zoom_btn_flash_mode
        if (zoom_btn_flash_mode == 0 and zoom_btn_flash > 0):
            zoom_btn_flash -= 1
            self.style.configure(
                'BarZoomButton.TFrame', background=color_tmpl.mix_color(self.color["background"], self.color["border_base"], zoom_btn_flash/4))
            self.style.configure(
                'BarZoomButton.TLabel', background=color_tmpl.mix_color(self.color["background"], self.color["border_base"], zoom_btn_flash/4))
        if (zoom_btn_flash_mode == 1 and zoom_btn_flash < 4):
            zoom_btn_flash += 1
            self.style.configure(
                'BarZoomButton.TFrame', background=color_tmpl.mix_color(self.color["background"], self.color["border_base"], zoom_btn_flash/4))
            self.style.configure(
                'BarZoomButton.TLabel', background=color_tmpl.mix_color(self.color["background"], self.color["border_base"], zoom_btn_flash/4))
        if (zoom_btn_flash_mode == 2):
            zoom_btn_flash = 4
            self.style.configure(
                'BarZoomButton.TFrame', background=color_tmpl.setlight(self.color["border_base"], 0.76))
            self.style.configure(
                'BarZoomButton.TLabel', background=color_tmpl.setlight(self.color["border_base"], 0.76))
    self.anim.append(animation)

    def enters_zoom(event):
        nonlocal zoom_btn_flash_mode
        zoom_btn_flash_mode = 1

    def leaves_zoom(event):
        nonlocal zoom_btn_flash_mode
        zoom_btn_flash_mode = 0

    def presshold_zoom(event):
        nonlocal zoom_btn_flash_mode
        zoom_btn_flash_mode = 2

    def press_zoom(event):
        nonlocal zoom_btn_flash_mode
        if (zoom_btn_flash_mode == 2):
            zoom_btn_flash_mode = 0
            self.zoom()
        else:
            zoom_btn_flash_mode = 0

    def update_icon(*args):
        if (self.fullscreen):
            label.configure(image=img_inzoom)
        else:
            label.configure(image=img_default)
    self.update_icon = update_icon
    zoom_btn.bind("<Enter>", enters_zoom)
    zoom_btn.bind("<Leave>", leaves_zoom)
    zoom_btn.bind("<Button-1>", presshold_zoom)
    zoom_btn.bind("<ButtonRelease-1>", press_zoom)
    label.bind("<Enter>", enters_zoom)
    label.bind("<Leave>", leaves_zoom)
    label.bind("<Button-1>", presshold_zoom)
    label.bind("<ButtonRelease-1>", press_zoom)


def iconic_btn_set(self, root):
    img_iconic = photoload.loadimg("iconic")

    self.style.configure(
        'BarIconicButton.TFrame', background=self.color["background"])
    self.style.configure(
        'BarIconicButton.TLabel', background=self.color["background"])

    iconic_btn = ttk.Frame(root, style='BarIconicButton.TFrame',
                           height=self.top_h, width=48)
    iconic_btn.pack(anchor="n", side='right')
    iconic_btn_flash = 0  # total 4f
    iconic_btn_flash_mode = 0
    label = ttk.Label(
        iconic_btn, style='BarIconicButton.TLabel', image=img_iconic, anchor=tkinter.CENTER)
    label.place(width=48, height=self.top_h, x=0, y=0)

    def animation():
        nonlocal iconic_btn_flash, iconic_btn_flash_mode
        if (iconic_btn_flash_mode == 0 and iconic_btn_flash > 0):
            iconic_btn_flash -= 1
            self.style.configure(
                'BarIconicButton.TFrame', background=color_tmpl.mix_color(self.color["background"], self.color["border_light"], iconic_btn_flash/4))
            self.style.configure(
                'BarIconicButton.TLabel', background=color_tmpl.mix_color(self.color["background"], self.color["border_light"], iconic_btn_flash/4))
        if (iconic_btn_flash_mode == 1 and iconic_btn_flash < 4):
            iconic_btn_flash += 1
            self.style.configure(
                'BarIconicButton.TFrame', background=color_tmpl.mix_color(self.color["background"], self.color["border_light"], iconic_btn_flash/4))
            self.style.configure(
                'BarIconicButton.TLabel', background=color_tmpl.mix_color(self.color["background"], self.color["border_light"], iconic_btn_flash/4))
        if (iconic_btn_flash_mode == 2):
            iconic_btn_flash = 4
            self.style.configure(
                'BarIconicButton.TFrame', background=color_tmpl.setlight(self.color["border_base"], 0.76))
            self.style.configure(
                'BarIconicButton.TLabel', background=color_tmpl.setlight(self.color["border_base"], 0.76))
    self.anim.append(animation)

    def enters_iconic(event):
        nonlocal iconic_btn_flash_mode
        iconic_btn_flash_mode = 1

    def leaves_iconic(event):
        nonlocal iconic_btn_flash_mode
        iconic_btn_flash_mode = 0

    def presshold_iconic(event):
        nonlocal iconic_btn_flash_mode
        iconic_btn_flash_mode = 2

    def press_iconic(event):
        nonlocal iconic_btn_flash_mode
        if (iconic_btn_flash_mode == 2):
            iconic_btn_flash_mode = 0
            self.iconify()
        else:
            iconic_btn_flash_mode = 0
    iconic_btn.bind("<Enter>", enters_iconic)
    iconic_btn.bind("<Leave>", leaves_iconic)
    iconic_btn.bind("<Button-1>", presshold_iconic)
    iconic_btn.bind("<ButtonRelease-1>", press_iconic)
    label.bind("<Enter>", enters_iconic)
    label.bind("<Leave>", leaves_iconic)
    label.bind("<Button-1>", presshold_iconic)
    label.bind("<ButtonRelease-1>", press_iconic)



from magictk.window import Window
from magictk.basicwindow import Tk
from magictk.button import Button, ButtonFill
from magictk.progressbar import ProgressBar
from magictk.checkbox import Checkbox, RadioGroup
from magictk.submenu import Menu, MenuObjs
from magictk.select import Select
from magictk.frame import Frame
from magictk.entry import Entry



import os
import subprocess
import sys
from typing import Optional, Tuple


def get_windows_workspace_size() -> Optional[Tuple[int, int, int, int]]:
    try:
        import win32api
        # 获取工作区（不包括任务栏）的尺寸
        work_area = win32api.GetMonitorInfo(
            win32api.MonitorFromPoint((0, 0))).get('Work')
        x, y, width, height = work_area
        width -= x
        height -= y
        return x, y, width, height
    except ImportError:
        print("pywin32 library is not installed.")
    except Exception as e:
        print(f"An error occurred: {e}")
    return (0, 0, 1920, 1002)


def get_gtk_workspace_size() -> Optional[Tuple[int, int, int, int]]:
    try:
        # if 1:
        import gi

        screen = gi.gdk.screen_get_default()

        # 获取屏幕上显示器的数量
        num_monitors = screen.get_n_monitors()

        # 获取每个显示器的几何信息，并计算工作区的大小
        workarea_width = 0
        workarea_height = 0

        for i in range(num_monitors):
            monitor_geometry = screen.get_monitor_geometry(i)
            monitor_x = monitor_geometry.x
            monitor_y = monitor_geometry.y
            monitor_width = monitor_geometry.width
            monitor_height = monitor_geometry.height

            workarea_width += monitor_width
            workarea_height += monitor_height

        print("工作区大小（宽度 x 高度）：{} x {}".format(workarea_width, workarea_height))
    except ImportError:
        print("pygtk (PyGObject  pygtk) library is not installed.")
    except:
        pass
    return None


def get_macos_workspace_size() -> Optional[Tuple[int, int, int, int]]:
    try:
        output = subprocess.check_output(
            ['system_profiler', 'SPDisplaysDataType']).decode('utf-8')
        lines = output.split('\n')
        for line in lines:
            if 'Resolution:' in line:
                dimensions = line.split(':')[1].strip().split('x')
                width = int(dimensions[0])
                height = int(dimensions[1])
                # system_profiler does not provide the top-left corner directly, assuming (0, 0)
                return 0, 0, width, height
    except subprocess.CalledProcessError:
        pass
    return None


def get_workspace_size() -> Optional[Tuple[int, int, int, int]]:
    if sys.platform.startswith('win'):
        return get_windows_workspace_size()
    elif sys.platform.startswith('darwin'):
        return get_macos_workspace_size()
    return get_gtk_workspace_size()



import json
import tkinter
import sys
from tkinter import ttk

from magictk import color_tmpl
from magictk import photoload
from magictk import fontconfig

transcolor = "#41342F"
usetrans = sys.platform.startswith('win')


class MenuObjs:
    def __init__(self):
        self.menu_lst = []
        self.menu_func = []
        self.menu_effect = []

    def first_action(self, menuobj):
        pass

    def addmenu(self, name, func, effect={}):
        self.menu_lst.append(name)
        self.menu_func.append(func)
        self.menu_effect.append(effect)


class Menu:
    __relh = 0
    __color_bd = "border_base"
    __color_bg = "background"
    __color_fg = "primary"
    __color_fg1 = "primary_light"
    __color_fg2 = "primary_light2"
    color = color_tmpl.default_color
    __close_mode = 0

    __last_highlight = 0
    __y_move = 0
    ___last_move = 1
    ___last_highlight = 0
    __load_y_size = 10
    ht = 12
    lt = 22
    ft = 34

    def __anim(self, *args):
        if (self.__close_mode == 0):
            if (self.__relh != self.h):
                if (self.h-self.__relh <= 1):
                    self.__relh = self.h
                elif (self.h-self.__relh > 32):
                    self.__relh += 32
                else:
                    self.__relh += (self.h-self.__relh) >> 1
                self.toplevel.geometry(
                    f"{self.w}x{self.__relh}+{self.x}+{self.y}")
        elif self.__close_mode == 1:
            if (self.__relh != 0):
                if (self.__relh-0 <= 1):
                    self.toplevel.destroy()
                    self.__close_mode = 2
                    return
                elif (self.__relh-0 > 32):
                    self.__relh -= 32
                else:
                    self.__relh -= (self.__relh-0) >> 1
                self.toplevel.geometry(
                    f"{self.w}x{self.__relh}+{self.x}+{self.y}")
        else:
            return -1
        if (self.___last_move != self.__y_move):
            n = 0
            for i in self.__itemsf:
                self.canvas.moveto(i, 24, n*self.ft+self.ht +
                                   self.__y_move+(self.ft-self.font[1]*2.5)//2)
                n += 1
            n = 0
            for i in self.__items:
                self.canvas.moveto(i, 2, n*self.ft+self.ht-4 +
                                   self.__y_move)
                n += 1
            self.___last_move = self.__y_move
        n = 0
        for i_tmp in range((-self.__real_y_move+self.__y_move)//self.ft, min((-self.__real_y_move+self.__y_move)//self.ft+len(self.__items), len(self.menuobj.menu_effect))):
            i = self.menuobj.menu_effect[i_tmp]
            if (len(i) != 0):
                self.canvas.itemconfigure(
                    self.__itemsf[n], justify=tkinter.LEFT, **i)
                self.canvas.moveto(self.__itemsf[n], 24, n*self.ft+self.ht +
                                   self.__y_move+(self.ft-self.font[1]*2.5)//2)
            else:
                self.canvas.itemconfigure(
                    self.__itemsf[n], justify=tkinter.LEFT, fill=self.color["regular_text"], font=self.font)
                self.canvas.moveto(self.__itemsf[n], 24, n*self.ft+self.ht +
                                   self.__y_move+(self.ft-self.font[1]*2.5)//2)
            n += 1

    def __mouse_move(self, event: tkinter.Event):
        y = event.y
        self.__last_highlight = max(
            -1, min(len(self.__items), int((y-self.ht-self.__y_move)//self.ft)))
        for i in self.__items:
            self.canvas.itemconfig(
                i, fill=self.color["background"])
        if -1 < self.__last_highlight < len(self.__items):
            self.canvas.itemconfig(
                self.__items[self.__last_highlight], fill=self.color["placeholder_light"])

    def close(self, *args):
        self.__close_mode = 1
        if sys.platform.startswith('win'):
            self.toplevel.unbind_all("<MouseWheel>")
        elif sys.platform.startswith('darwin'):
            self.toplevel.unbind_all("<MouseWheel>")
        else:
            self.toplevel.unbind_all(
                "<Button-4>")
            self.toplevel.unbind_all(
                "<Button-5>")
        if (self.__closecallback is not None):
            self.__closecallback(self)

    def __init__(self, root, menuobj: MenuObjs, w=200, h=300, x=100, y=300, color_list: dict = None, closeonleave=True, fontsize=10, closecallback=None):
        global use_font
        use_font = fontconfig.getfont()
        self.__closecallback = closecallback
        self.font = (use_font, fontsize)
        self.__items = []
        self.__itemsf = []
        self.__itemsid = []
        self.__real_y_move = 0
        self.menuobj = menuobj
        self.w = w
        self.h = h
        self.x = x
        self.y = y
        if (color_list is not None):
            self.color = color_list
        self.toplevel = tkinter.Toplevel(bg=transcolor)
        self.toplevel.wm_attributes('-topmost', True)
        self.toplevel.overrideredirect(True)
        self.toplevel.geometry(f"{self.w}x{self.__relh}+{self.x}+{self.y}")
        if (usetrans):
            self.toplevel.wm_attributes('-transparentcolor', transcolor)
        self.root = root
        self.canvas = tkinter.Canvas(
            self.toplevel, bg=self.color["background"],  highlightthickness=0, width=self.w+1, height=self.h+1)
        self.canvas.place(x=-1, y=-1)
        self.root.anim.append(self.__anim)
        self.toplevel.focus()
        self.__draw_menu()
        self.__draws()

        if (closeonleave):
            self.toplevel.bind("<FocusOut>", self.close)
            if (not usetrans):
                self.toplevel.bind("<Leave>", self.close)

        self.canvas.bind("<Motion>", self.__mouse_move)
        self.__bind_scroll()
        self.canvas.bind("<ButtonRelease-1>", self.__click)
        self.toplevel.bind_all("<Escape>", self.close)
        self.toplevel.bind_all("<Return>", self.__click)

    def __draw_menu(self):
        n = 0
        for i_tmp in range(0, min(self.__load_y_size, len(self.menuobj.menu_lst))):
            i = self.menuobj.menu_lst[i_tmp]
            self.__items.append(self.canvas.create_rectangle(
                2, n*self.ft+self.ht, self.w, (n+1)*self.ft+self.ht, fill=self.color["background"], width=0))
            self.__itemsf.append(self.canvas.create_text(
                10, n*self.ft+self.ht, text=i, font=self.font, justify=tkinter.LEFT, fill=self.color["regular_text"]))
            self.__itemsid.append(n)
            n += 1

    def __draw_corner(self, r_x, r_y, x, y, **kwargs):
        border_info = json.loads(photoload.loadres("menuborder"))
        y_n = 0
        for i in border_info:
            x_n = 0
            for j in i:
                if (r_x == 0):
                    px = x+x_n+1
                else:
                    px = x+4-x_n-1
                if (r_y == 0):
                    py = y+y_n+1
                else:
                    py = y+4-y_n-1
                if (j < 0):
                    lcolor = -j
                else:
                    lcolor = j
                if (lcolor == 255):
                    x_n += 1
                    continue
                if (j < 0):
                    if (usetrans):
                        g_color = transcolor
                    else:
                        g_color = self.color["background"]
                else:
                    g_color = color_tmpl.mix_color(
                        self.color["background"], self.color[self.__color_bd], int((1-lcolor/255)*1000)/1000)
                self.canvas.create_rectangle(
                    px, py, px, py, width=0, fill=g_color)
                x_n += 1
            y_n += 1

    def __draws(self):
        self.__draw_corner(0, 0, 0, 0)
        self.__draw_corner(1, 0, self.w-4+1, 0)
        self.__draw_corner(0, 1, 0, self.h-4+1)
        self.__draw_corner(1, 1, self.w-4+1, self.h-4+1)
        self.canvas.create_line(
            1, 5, 1, self.h-3, width=1, fill=self.color[self.__color_bd])
        self.canvas.create_line(
            5, 1, self.w-3, 1, width=1, fill=self.color[self.__color_bd])
        self.canvas.create_line(
            self.w, 5, self.w, self.h-3, width=1, fill=self.color[self.__color_bd])
        self.canvas.create_line(
            5, self.h, self.w-3, self.h, width=1, fill=self.color[self.__color_bd])

    def __bind_scroll(self):
        def scrollwheel(event: tkinter.Event, setdelta=None):
            if (len(self.menuobj.menu_lst) <= 8):
                return
            if (setdelta is None):
                delta = event.delta
            else:
                delta = setdelta
            if sys.platform.startswith('win'):
                delta //= 20
            if delta > 0:
                if (self.__real_y_move != 0):
                    self.__y_move += delta
                self.__real_y_move = min(0, self.__real_y_move+delta)
            else:
                if (self.__real_y_move != -(len(self.menuobj.menu_lst)*self.ft-self.h+8)):
                    self.__y_move += delta
                self.__real_y_move = max(self.__real_y_move+delta,
                                         -(len(self.menuobj.menu_lst)*self.ft-self.h+8))

            if (self.__y_move > 0):
                self.__y_move -= self.ft
                self.__items.insert(0, self.__items.pop())
                self.__itemsf.insert(0, self.__itemsf.pop())
                self.__itemsid.insert(0, self.__itemsid.pop()-10)
                self.__mouse_move(event)
                if (self.__itemsid[0] >= 0):
                    self.canvas.itemconfigure(
                        self.__itemsf[0], text=self.menuobj.menu_lst[self.__itemsid[0]])
            elif (self.__y_move < -self.ft+1):
                self.__y_move += self.ft
                self.__items.append(self.__items.pop(0))
                self.__itemsf.append(self.__itemsf.pop(0))
                self.__itemsid.append(self.__itemsid.pop(0)+10)
                self.__mouse_move(event)
                if (self.__itemsid[-1] < len(self.menuobj.menu_lst)):
                    self.canvas.itemconfigure(
                        self.__itemsf[-1], text=self.menuobj.menu_lst[self.__itemsid[-1]])

        if sys.platform.startswith('win'):
            self.toplevel.bind_all("<MouseWheel>", scrollwheel)
        elif sys.platform.startswith('darwin'):
            self.toplevel.bind_all("<MouseWheel>", scrollwheel)
        else:
            self.toplevel.bind_all(
                "<Button-4>", lambda event: scrollwheel(event, 8))
            self.toplevel.bind_all(
                "<Button-5>", lambda event: scrollwheel(event, -8))

    def __click(self, event):
        click_item = max(
            -1, min(len(self.menuobj.menu_func), int((event.y-self.ht-self.__real_y_move)//self.ft)))
        if (click_item == -1 or self.__last_highlight >= len(self.menuobj.menu_func)):
            return
        self.close()
        self.menuobj.menu_func[click_item](self, click_item)



import colorsys

default_color = {
    "primary": "#409EFF",
    "primary_light": "#C6E2FF",
    "primary_light2": "#ECF5FF",
    "primary_light3": "#79BBFF",
    "primary_dark": "#337ECC",
    "success": "#67C23A",
    "success_light": "#E1F3D8",
    "success_light2": "#F0F9EB",
    "success_light3": "#95D475",
    "success_dark": "#529B2E",
    "warning": "#E6A23C",
    "warning_light": "#FAECD8",
    "warning_light2": "#E6A23C",
    "warning_light3": "#EEBE77",
    "warning_dark": "#B88230",
    "danger": "#F56C6C",
    "danger_light": "#FDE2E2",
    "danger_light2": "#FEF0F0",
    "danger_light3": "#F89898",
    "danger_dark": "#C45656",
    "info": "#909399",
    "info_light": "#E9E9EB",
    "info_light2": "#F4F4F5",
    "info_light3": "#B1B3B8",
    "info_dark": "#73767A",
    "primary_text": "#303133",
    "regular_text": "#606266",
    "secondary_text": "#909399",
    "placeholder": "#C0C4CC",
    "placeholder_light": "#F5F7FA",
    "border_base": "#DCDFE6",
    "border_light": "#E4E7ED",
    "background": "#FFFFFF"
}


def hex2rgb(color: str):
    return (int(color[1:3], 16), int(color[3:5], 16), int(color[5:7], 16))


def rgb2hex(r, g, b):
    return f"#{str(hex(int(r)))[2:].upper():0>2}{str(hex(int(g)))[2:].upper():0>2}{str(hex(int(b)))[2:].upper():0>2}"


def hex2hls(color: str):
    r, g, b = hex2rgb(color)
    return colorsys.rgb_to_hls(r/255, g/255, b/255)


def hls2hex(h, l, s):
    r, g, b = colorsys.hls_to_rgb(h, l, s)
    return rgb2hex(r*255, g*255, b*255)


def setlight(color: str, light: float):
    h, l, s = hex2hls(color)
    return hls2hex(h, light, s)


def mix_color(c1: str, c2: str, alpha: float):
    r2, g2, b2 = hex2rgb(c1)
    r1, g1, b1 = hex2rgb(c2)
    r3 = int(r1*alpha+r2*(1-alpha))
    g3 = int(g1*alpha+g2*(1-alpha))
    b3 = int(b1*alpha+b2*(1-alpha))
    return rgb2hex(r3, g3, b3)



import json
import tkinter
import sys
from tkinter import ttk
from magictk import color_tmpl
from magictk import photoload
from magictk import fontconfig
from magictk import button


class Entry:
    color = color_tmpl.default_color
    __fill_obj = []
    __fill_func = []
    __fill_gc = []
    __fill_fc = []
    __fill_hc = []
    hover_mode = 0.0
    __is_hover = 0
    __flash_t = 0
    max_flash = 6
    __anim_obj_id = -1
    __color_bd = "border_base"
    __color_bg = "background"
    __color_fg = "primary"
    __color_fg1 = "primary"
    __color_fg2 = "background"
    text = "Input"
    input_text = ""

    def __draw_corner(self, r_x, r_y, x, y, **kwargs):
        border_info = json.loads(photoload.loadres("buttonborder"))
        y_n = 0
        for i in border_info:
            x_n = 0
            for j in i:
                if (r_x == 0):
                    px = x+x_n+1
                else:
                    px = x+4-x_n-1
                if (r_y == 0):
                    py = y+y_n+1
                else:
                    py = y+4-y_n-1
                if (j < 0):
                    lcolor = -j
                else:
                    lcolor = j
                g_color = color_tmpl.mix_color(
                    self.color["background"], self.color[self.__color_bd], int((1-lcolor/255)*1000)/1000)
                if (j < 0):
                    f_color = color_tmpl.mix_color(
                        self.color[self.__color_fg2], self.color[self.__color_fg1], int((1-lcolor/255)*1000)/1000)
                else:
                    f_color = color_tmpl.mix_color(
                        self.color["background"], self.color[self.__color_fg1], int((1-lcolor/255)*1000)/1000)
                if (j < 0):
                    h_color = color_tmpl.mix_color(
                        self.color[self.__color_fg2], self.color[self.__color_fg], int((1-lcolor/255)*1000)/1000)
                else:
                    h_color = color_tmpl.mix_color(
                        self.color["background"], self.color[self.__color_fg], int((1-lcolor/255)*1000)/1000)

                obj = self.canvas.create_rectangle(
                    px, py, px, py, width=0, fill=g_color)

                def update_color(obj, g_color, f_color, h_color):
                    if (self.__is_hover == 2):
                        self.canvas.itemconfig(
                            obj, fill=h_color)
                    else:
                        self.canvas.itemconfig(
                            obj, fill=color_tmpl.mix_color(g_color, f_color, self.hover_mode))

                self.__fill_func.append(update_color)
                self.__fill_gc.append(g_color)
                self.__fill_fc.append(f_color)
                self.__fill_hc.append(h_color)
                self.__fill_obj.append(obj)
                x_n += 1
            y_n += 1

    def __update_color(self):
        n = 0
        for i in self.__fill_func:
            i(self.__fill_obj[n], self.__fill_gc[n],
              self.__fill_fc[n], self.__fill_hc[n])
            n += 1

    def __init__(self, master=None, root_anim=None, w=200, h=30, text="Input",  color_list: dict = None, items=[]):
        global use_font
        use_font = fontconfig.getfont()
        self.input_text = ""
        self.items = items
        self.w = max(120, w)
        self.h = max(30, h)
        self.text = text
        self.choose = -1
        self.__master = master
        if (color_list is not None):
            self.color = color_list
        if (root_anim == None):
            self.root = master.root
        else:
            self.root = root_anim

        self.frames = tkinter.Frame(
            master, width=self.w, height=self.h, bg=self.color["background"])
        self.canvas = tkinter.Canvas(
            self.frames,
            bg=self.color["background"],
            width=self.w,
            height=self.h,
            borderwidth=0,
            bd=0,
            highlightcolor=self.color["background"],
            highlightthickness=0
        )
        self.canvas.place(x=0, y=0, relwidth=1.0, relheight=1.0)
        self.__input_textvar = tkinter.StringVar()
        self.inputobj = tkinter.Entry(
            self.frames,
            bg=self.color["background"],
            fg=self.color["regular_text"],
            highlightthickness=0,
            bd=0,
            font=(use_font, 10),
            textvariable=self.__input_textvar
        )
        self.inputobj.place(x=10, y=6, width=self.w-20, height=self.h-12)
        self.show_label = tkinter.Label(self.frames,
                                        text=self.text,
                                        bg=self.color["background"],
                                        fg=self.color["secondary_text"],
                                        anchor="w",
                                        font=(use_font, 10)
                                        )
        self.show_label.place(x=10, y=6, width=self.w-20, height=self.h-12)
        self.packed = True

        self.__draw()
        self.__update_color()
        self.__bind_event()
        self.bind_anim()

    def pack(self, *args, **kwargs):
        self.frames.pack(*args, **kwargs)

    def guid(self, *args, **kwargs):
        self.frames.guid(*args, **kwargs)

    def place(self, *args, **kwargs):
        self.frames.place(*args, **kwargs)

    def __draw(self):
        self.__draw_corner(0, 0, 0, 0)
        self.__draw_corner(1, 0, self.w-4, 0)
        self.__draw_corner(0, 1, 0, self.h-5)
        self.__draw_corner(1, 1, self.w-4, self.h-5)

        def update_color(obj, g_color, f_color, h_color):
            if (self.__is_hover == 2):
                self.canvas.itemconfig(
                    obj, fill=h_color)
            else:
                self.canvas.itemconfig(
                    obj, fill=color_tmpl.mix_color(g_color, f_color, self.hover_mode))
        self.__fill_fc.append(self.color[self.__color_fg1])
        self.__fill_gc.append(self.color[self.__color_bd])
        self.__fill_hc.append(self.color[self.__color_fg])
        self.__fill_func.append(update_color)
        self.__fill_obj.append(self.canvas.create_line(
            1, 5, 1, self.h-5, width=1, fill=self.color[self.__color_bd]))
        self.__fill_fc.append(self.color[self.__color_fg1])
        self.__fill_gc.append(self.color[self.__color_bd])
        self.__fill_hc.append(self.color[self.__color_fg])
        self.__fill_func.append(update_color)
        self.__fill_obj.append(self.canvas.create_line(
            5, 1, self.w-4, 1, width=1, fill=self.color[self.__color_fg1]))
        self.__fill_fc.append(self.color[self.__color_fg1])
        self.__fill_gc.append(self.color[self.__color_bd])
        self.__fill_hc.append(self.color[self.__color_fg])
        self.__fill_func.append(update_color)
        self.__fill_obj.append(self.canvas.create_line(
            self.w-1, 5, self.w-1, self.h-5, width=1, fill=self.color[self.__color_fg1]))
        self.__fill_fc.append(self.color[self.__color_fg1])
        self.__fill_gc.append(self.color[self.__color_bd])
        self.__fill_hc.append(self.color[self.__color_fg])
        self.__fill_func.append(update_color)
        self.__fill_obj.append(self.canvas.create_line(
            5, self.h-2, self.w-4, self.h-2, width=1, fill=self.color[self.__color_fg1]))

        self.__fill_fc.append(self.color[self.__color_fg2])
        self.__fill_gc.append(self.color["background"])
        self.__fill_hc.append(self.color[self.__color_fg2])
        self.__fill_func.append(update_color)
        self.__fill_obj.append(self.canvas.create_rectangle(
            2, 5, self.w-1, self.h-5, width=0, fill=self.color[self.__color_fg2]))

        self.__fill_fc.append(self.color[self.__color_fg2])
        self.__fill_gc.append(self.color["background"])
        self.__fill_hc.append(self.color[self.__color_fg2])
        self.__fill_func.append(update_color)
        self.__fill_obj.append(self.canvas.create_rectangle(
            5, 2, self.w-4, self.h-2, width=0, fill=self.color[self.__color_fg2]))

        self.__text_obj = self.canvas.create_text(
            20, int(self.h/2), text=self.text, font=(use_font, 10), justify="left")
        self.__fill_fc.append(self.color["placeholder"])
        self.__fill_gc.append(self.color["placeholder"])
        self.__fill_hc.append(self.color["placeholder"])
        self.__fill_func.append(update_color)
        self.__fill_obj.append(self.__text_obj)
        self.canvas.moveto(self.__text_obj, 16, self.h//2-10)

    def bind_anim(self):
        def anim_magictk():
            if (self.__is_hover == 1 and self.__flash_t < self.max_flash):
                self.__flash_t += (1 if (len(self.root.anim) > 6) else 1)
                self.__flash_t = min(self.__flash_t, self.max_flash)
                self.hover_mode = self.__flash_t/self.max_flash
                self.__update_color()
            elif (self.__is_hover == 0 and self.__flash_t > 0):
                self.__flash_t -= (1 if (len(self.root.anim) > 6) else 1)
                self.__flash_t = max(self.__flash_t, 0)
                self.hover_mode = self.__flash_t/self.max_flash
                self.__update_color()

        def anim_normal(*args):
            if (self.__is_hover == 1 and self.__flash_t < self.max_flash):
                self.__flash_t += 1
                self.hover_mode = self.__flash_t/self.max_flash
                self.__update_color()
            elif (self.__is_hover == 0 and self.__flash_t > 0):
                self.__flash_t -= 1
                self.hover_mode = self.__flash_t/self.max_flash
                self.__update_color()
            self.root.after(anim_normal, 16)
        try:
            self.root.anim == 0
        except:
            self.root.after(anim_normal, 16)
        else:
            if (anim_magictk not in self.root.anim):
                self.root.anim.append(anim_magictk)
                self.__anim_obj_id = self.root.anim[-1]

    def __bind_event(self):
        def closecallback(obj):
            if (self.__is_hover == 1):
                if sys.platform.startswith("linux") and self.root.FRAME_INFO == "Custom":
                    self.top.destroy()
                self.__is_hover = 0
                self.canvas.focus_force()

        def pressrelease_v(event: tkinter.Event):
            if (self.__is_hover == 1):
                pass
                # closecallback(event)
            else:
                self.__is_hover = 1
                # what's fuck?
                if sys.platform.startswith("linux") and self.root.FRAME_INFO == "Custom":
                    self.top = tkinter.Toplevel()
                    self.top.geometry(
                        f"{1}x{1}+{10000}+{10000}")
                    self.__top_entry = ttk.Entry(self.top, width=self.w)
                    self.__top_entry.place(
                        x=0, y=0, width=self.w, height=self.h)
                self.root.update()
                self.inputobj.focus_force()
            self.__update_color()

        def update_text(*args):
            self.input_text = self.__input_textvar.get()
            if (self.input_text == ""):
                if (self.packed == False):
                    self.show_label.place(
                        x=10, y=6, width=self.w-20, height=self.h-12)
                    self.packed = True
            else:
                if (self.packed):
                    self.show_label.place_forget()
                    self.packed = False
        self.frames.bind("<ButtonRelease-1>", pressrelease_v)
        self.canvas.bind("<ButtonRelease-1>", pressrelease_v)
        self.inputobj.bind("<ButtonRelease-1>", pressrelease_v)
        self.show_label.bind("<ButtonRelease-1>", pressrelease_v)
        self.__input_textvar.trace_add("write", update_text)
        if (sys.platform.startswith("linux")):
            self.frames.bind("<Leave>", closecallback)
        else:
            self.inputobj.bind_all("<FocusOut>", closecallback)




from tkinter import font as tkfont

font = None


def set_font():
    font_family = ["Helvetica Neue", "Helvetica", "PingFang SC", "Hiragino Sans GB",
                   "Microsoft YaHei",
                   "微软雅黑",
                   "Arial", "sans-serif"]
    t_family = tkfont.families(root=None, displayof=None)
    for i in font_family:
        if i in t_family:
            use_font = i
            break
    else:
        print("Unknown font")
    return use_font


def getfont():
    global font
    if (font is None):
        font = set_font()
    return font



import json
import tkinter
from tkinter import ttk
from magictk import color_tmpl
from magictk import photoload
from magictk import fontconfig
from magictk import button


class Checkbox(button.ButtonFill):
    hover_mode = 0.0
    ishover = 0
    _flash_t = 0
    max_flash = 4
    _anim_obj_id = -1
    checked = False
    text = "ButtonFill"
    _group_id = 0
    disable_unhover = 0
    def callback(*args): return None

    def _draw_corner(self, r_x, r_y, x, y, **kwargs):
        border_info = json.loads(photoload.loadres("checkboxborder"))
        y_n = 0
        for i in border_info:
            x_n = 0
            for j in i:
                if (r_x == 0):
                    px = x+x_n+1
                else:
                    px = x+2-x_n-1
                if (r_y == 0):
                    py = y+y_n+1
                else:
                    py = y+2-y_n-1
                if (j < 0):
                    lcolor = 255+j
                else:
                    lcolor = 255-j
                if (j < 0):
                    g_color = color_tmpl.mix_color(
                        self.color[self._color_bg], self.color[self._color_bd], int((1-lcolor/255)*1000)/1000)
                else:
                    g_color = color_tmpl.mix_color(
                        self.color["background"], self.color[self._color_bd], int((1-lcolor/255)*1000)/1000)
                if (j < 0):
                    f_color = color_tmpl.mix_color(
                        self.color[self._color_fg1], self.color[self._color_fg1], int((1-lcolor/255)*1000)/1000)
                else:
                    f_color = color_tmpl.mix_color(
                        self.color["background"], self.color[self._color_fg1], int((1-lcolor/255)*1000)/1000)
                if (j < 0):
                    h_color = color_tmpl.mix_color(
                        self.color[self._color_fg2], self.color[self._color_fg2], int((1-lcolor/255)*1000)/1000)
                else:
                    h_color = color_tmpl.mix_color(
                        self.color["background"], self.color[self._color_fg2], int((1-lcolor/255)*1000)/1000)

                obj = self.canvas.create_rectangle(
                    px, py, px, py, width=0, fill=g_color)

                def update_color(obj, g_color, f_color, h_color):
                    if (self.ishover == 2):
                        self.canvas.itemconfig(
                            obj, fill=h_color)
                    elif (self._flash_t <= self.max_flash):
                        self.canvas.itemconfig(
                            obj, fill=color_tmpl.mix_color(g_color, f_color, self.hover_mode))

                self._fill_func.append(update_color)
                self._fill_gc.append(g_color)
                self._fill_fc.append(f_color)
                self._fill_hc.append(h_color)
                self._fill_obj.append(obj)
                x_n += 1
            y_n += 1

    def _draw_img(self, x, y, **kwargs):
        border_info = json.loads(photoload.loadres("checkbox"))
        y_n = 0
        for i in border_info:
            x_n = 0
            for j in i:
                px = x+x_n+1
                py = y+y_n+1
                if (j < 0):
                    lcolor = 255+j
                else:
                    lcolor = 255-j
                g_color = self.color[self._color_fg]
                f_color = color_tmpl.mix_color(
                    self.color["background"], self.color[self._color_fg], int((1-lcolor/255)*1000)/1000)
                h_color = color_tmpl.mix_color(
                    self.color["background"], self.color[self._color_fg2], int((1-lcolor/255)*1000)/1000)

                obj = self.canvas.create_rectangle(
                    px, py, px, py, width=0, fill=g_color)

                def update_color(obj, g_color, f_color, h_color):
                    if (self._flash_t > self.max_flash):
                        self.canvas.itemconfig(
                            obj, fill=color_tmpl.mix_color(g_color, f_color, (self._flash_t-self.max_flash)/self.max_flash))
                    else:
                        self.canvas.itemconfig(
                            obj, fill=color_tmpl.mix_color(self.color["background"], self.color[self._color_fg1], self.hover_mode))

                self._fill_func.append(update_color)
                self._fill_gc.append(g_color)
                self._fill_fc.append(f_color)
                self._fill_hc.append(h_color)
                self._fill_obj.append(obj)
                x_n += 1
            y_n += 1

    def _update_color(self):
        n = 0
        for i in self._fill_func:
            i(self._fill_obj[n], self._fill_gc[n],
              self._fill_fc[n], self._fill_hc[n])
            n += 1

    def __init__(self, master=None, root_anim=None, color_type="primary", w=80, h=16, text="Button", color_list: dict = None, group: None = None):
        global use_font
        use_font = fontconfig.getfont()
        self._color_bd = "border_base"
        self._color_bg = "background"
        self._color_fg = "primary"
        self._color_fg1 = "primary"
        self._color_fg2 = "primary_dark"
        super().__init__(master=master, root_anim=root_anim, w=w, h=h,
                         text=text, color_list=color_list, color_type=color_type, _dis_color=True)
        if (group is not None):
            group._add_checkbox(self)

    def pack(self, *args, **kwargs):
        self.canvas.pack(*args, **kwargs)

    def guid(self, *args, **kwargs):
        self.canvas.guid(*args, **kwargs)

    def place(self, *args, **kwargs):
        self.canvas.place(*args, **kwargs)

    def _draw(self):
        self._draw_corner(0, 0, 0, 0)
        self._draw_corner(1, 0, 12, 0)
        self._draw_corner(0, 1, 0, 12)
        self._draw_corner(1, 1, 12, 12)

        def update_color(obj, g_color, f_color, h_color):
            if (self.ishover == 2):
                self.canvas.itemconfig(
                    obj, fill=h_color)
            elif (self._flash_t <= self.max_flash):
                self.canvas.itemconfig(
                    obj, fill=color_tmpl.mix_color(g_color, f_color, self.hover_mode))
        self._fill_fc.append(self.color[self._color_fg1])
        self._fill_gc.append(self.color[self._color_bd])
        self._fill_hc.append(self.color[self._color_fg2])
        self._fill_func.append(update_color)
        self._fill_obj.append(self.canvas.create_line(
            1, 3, 1, 12, width=1, fill=self.color[self._color_bd]))
        self._fill_fc.append(self.color[self._color_fg1])
        self._fill_gc.append(self.color[self._color_bd])
        self._fill_hc.append(self.color[self._color_fg2])
        self._fill_func.append(update_color)
        self._fill_obj.append(self.canvas.create_line(
            3, 1, 12, 1, width=1, fill=self.color[self._color_fg1]))
        self._fill_fc.append(self.color[self._color_fg1])
        self._fill_gc.append(self.color[self._color_bd])
        self._fill_hc.append(self.color[self._color_fg2])
        self._fill_func.append(update_color)
        self._fill_obj.append(self.canvas.create_line(
            13, 3, 13, 12, width=1, fill=self.color[self._color_fg1]))
        self._fill_fc.append(self.color[self._color_fg1])
        self._fill_gc.append(self.color[self._color_bd])
        self._fill_hc.append(self.color[self._color_fg2])
        self._fill_func.append(update_color)
        self._fill_obj.append(self.canvas.create_line(
            3, 13, 12, 13, width=1, fill=self.color[self._color_fg1]))

        self._fill_fc.append(self.color[self._color_fg])
        self._fill_gc.append(self.color["background"])
        self._fill_hc.append(self.color[self._color_fg2])
        self._fill_func.append(update_color)
        self._fill_obj.append(self.canvas.create_rectangle(
            2, 3, 13, 12, width=0, fill=self.color[self._color_fg2]))

        self._fill_fc.append(self.color[self._color_fg])
        self._fill_gc.append(self.color["background"])
        self._fill_hc.append(self.color[self._color_fg2])
        self._fill_func.append(update_color)
        self._fill_obj.append(self.canvas.create_rectangle(
            3, 2, 12, 13, width=0, fill=self.color[self._color_fg2]))

        self._draw_img(2, 2)

        self._fill_fc.append(self.color[self._color_fg])
        self._fill_gc.append(self.color["regular_text"])
        self._fill_hc.append("#FFFFFF")
        self._fill_func.append(update_color)
        self._fill_obj.append(
            self.canvas.create_text(14+int((self.w-14)/2), 6, text=self.text, font=(use_font, 10)))

    def bind_anim(self):
        def anim_magictk():
            if (self.ishover == 1 and self._flash_t < self.max_flash*2):
                self._flash_t += (1 if (len(self.root.anim) > 6) else 1)
                self._flash_t = min(self._flash_t, self.max_flash*2)
                self.hover_mode = self._flash_t/self.max_flash
                self._update_color()
            elif (self.ishover == 0 and self._flash_t > 0):
                self._flash_t -= (1 if (len(self.root.anim) > 6) else 1)
                self._flash_t = max(self._flash_t, 0)
                self.hover_mode = self._flash_t/self.max_flash
                self._update_color()
            elif (self._is_hover == 0 and self._flash_t <= 0):
                return -1

        def anim_normal(*args):
            if (self.ishover == 1 and self._flash_t < self.max_flash*2):
                self._flash_t += 1
                self.hover_mode = self._flash_t/self.max_flash
                self.__update_color()
            elif (self.ishover == 0 and self._flash_t > 0):
                self._flash_t -= 1
                self.hover_mode = self._flash_t/self.max_flash
                self.__update_color()
            self.root.after(anim_normal, 16)

        try:
            self.root.anim == 0
        except:
            self.root.after(anim_normal, 16)
        else:
            if (anim_magictk not in self.root.anim):
                self.root.anim.append(anim_magictk)
                self._anim_obj_id = self.root.anim[-1]

    def _bind_event(self):

        def press_v(*args):
            self.bind_anim()
            if (self.ishover == 1):
                if (self.disable_unhover == 0):
                    self.ishover = 0
            else:
                self.ishover = 1
                self.callback(self)
            self._update_color()
        self.canvas.bind("<Button-1>", press_v)

        def pressrelease_v(*args):
            self._update_color()
        self.canvas.bind("<ButtonRelease-1>", pressrelease_v)


class RadioGroup:
    value = 1
    __g_list = []

    def __change(self, btn):
        for i in self.__g_list:
            self.value = btn._group_id
            if (i != btn):
                i.ishover = 0
            else:
                i.ishover = 1

    def _add_checkbox(self, cb: Checkbox):
        self.__g_list.append(cb)
        cb.callback = self.__change
        cb._group_id = len(self.__g_list)
        cb.max_flash = 3
        cb.disable_unhover = 1
        if (len(self.__g_list) == 1):
            cb.ishover = 1



import time
import tkinter
import os
from tkinter import ttk
from magictk import color_tmpl
from magictk import _window_ctl
from magictk import _window_size
from magictk import workspace
from magictk import photoload
import sys

WIN_INF = -10000


class Tk(ttk.Frame):
    x = 200
    y = 200
    w = 500
    h = 350
    color = color_tmpl.default_color
    anim = []
    FRAME_INFO = "Normal"
    __break_win = True

    def __load_color(self) -> None:
        self.main_tk.configure(bg=self.color["background"])
        self.style.configure(
            'Test1.TFrame', background="#FF0000")
        self.style.configure(
            'Test2.TFrame', background="#00FF00")
        self.style.configure(
            'Test3.TFrame', background="#0000FF")
        self.style.configure(
            'Main.TFrame', background=self.color["background"])
        self.style.configure(
            'Root.TFrame', background=self.color["background"])
        self.style.configure(
            'Splitline.TFrame', background=self.color["border_light"])
        self.style.configure(
            'BarExitButton.TFrame', background=self.color["background"])
        self.style.configure(
            'LabBarZoomButton.TLabel', background=self.color["background"])
        self.style.configure(
            'LabBarIcon.TLabel', background=self.color["background"])
        self.style.configure(
            'LabTitle.TLabel', background=self.color["background"], fg=self.color["primary_text"])
        self.style.configure(
            'BarIconicButton.TFrame', background=self.color["background"])
        self.style.configure(
            'Size.TFrame', background=self.color["background"])
        self.style.configure(
            'LabelTitle.TFrame', background=self.color["background"])

    def __init__(self, w=500, h=350, title="MagicTk", color_list: dict = None) -> None:
        self.root = self
        self.title = title
        self.w = w
        self.h = h
        if (color_list is not None):
            self.color = color_list
        self.main_tk = tkinter.Tk()
        self.style = ttk.Style()
        self.__load_color()
        ttk.Widget.__init__(self, self.main_tk, "ttk::frame", {
                            "style": "Main.TFrame"})
        self.main_tk.title(title)
        self.place(x=0, y=0, relwidth=1, relheight=1)
        self.main_tk.geometry(
            f"{self.w}x{self.h}+{self.x}+{self.y}")

    def mainloop(self) -> None:
        t_start = time.time()
        while (self.__break_win):
            delta_t = time.time()-t_start
            if (delta_t > 0.02):  # flash animation
                t_start = time.time()
                n = 0
                for i in self.anim:
                    if (i is not None):
                        retn = i()
                        if (retn == -1):
                            self.anim[n] = None
                    n += 1
                self.anim = [i for i in self.anim if i is not None]
            else:
                pass
            self.main_tk.update()
        self.main_tk.destroy()
        self.__break_win = None

    def quit(self) -> None:
        self.__break_win = False



import pickle
import os
import tkinter
import base64
image_all = None
load_cache = {}


def fresh_image():
    global image_all
    if (image_all == None):
        with open(os.path.dirname(__file__).replace("\\", "/")+"/res.pickle", "rb") as file:
            image_all = pickle.load(file)


fresh_image()


def loadimg(imgid: str):
    global image_all
    if imgid not in load_cache:
        load_cache[imgid] = tkinter.PhotoImage(
            data=base64.b64decode(image_all[imgid]))
    return load_cache[imgid]


def loadres(imgid: str):
    global image_all
    if imgid not in load_cache:
        load_cache[imgid] = base64.b64decode(image_all[imgid])
    return load_cache[imgid]



import json
import tkinter
import sys
from tkinter import ttk
from tkinter import font as tkfont

from magictk import color_tmpl
from magictk import photoload
from magictk import submenu
from magictk import fontconfig
from magictk import button


class Select(button.Button):
    color = color_tmpl.default_color
    _fill_obj = []
    _fill_func = []
    _fill_gc = []
    _fill_fc = []
    _fill_hc = []
    hover_mode = 0.0
    _is_hover = 0
    _flash_t = 0
    max_flash = 6
    _anim_obj_id = -1

    def _draw_arrow(self, x, y, **kwargs):
        border_info = self.__arrow_json[0]
        self.__arrow_list = []
        y_n = 0
        for i in border_info:
            x_n = 0
            self.__arrow_list.append([])
            for j in i:
                px = x+x_n+1
                py = y+y_n+1
                lcolor = j
                g_color = color_tmpl.mix_color(
                    self.color["background"], self.color[self._color_bd], int((1-lcolor/255)*1000)/1000)

                obj = self.canvas.create_rectangle(
                    px, py, px, py, width=0, fill=g_color)
                self.__arrow_list[-1].append(obj)

                x_n += 1
            y_n += 1

    def __init__(self, master=None, root_anim=None, w=200, h=30, text="Select",  color_list: dict = None, items=[]):
        global use_font
        use_font = fontconfig.getfont()
        self.__arrow_json = json.loads(photoload.loadres("selectarrow"))
        self.items = items
        self._color_bd = "border_base"
        self._color_bg = "background"
        self._color_bg1 = "background"
        self._color_fg = "primary"
        self._color_fg1 = "primary"
        self._color_fg2 = "background"
        self._color_fg3 = "background"
        self._color_text = None
        super().__init__(master=master, root_anim=root_anim, w=w,
                         h=h, text=text,  color_list=color_list, _set_defaultcolor=True)
        self.__menuobj = submenu.MenuObjs()
        self.__last = 0
        for i in self.items:
            self.__menuobj.addmenu(i, self._callback_menu)

    def _draw(self):
        super()._draw(True)

        def update_color(obj, g_color, f_color, h_color):
            if (self._is_hover == 2):
                self.canvas.itemconfig(
                    obj, fill=h_color)
            else:
                self.canvas.itemconfig(
                    obj, fill=color_tmpl.mix_color(g_color, f_color, self.hover_mode))
        self.__text_obj = self.canvas.create_text(
            20, int(self.h/2), text=self.text, font=(use_font, 10), justify="left")
        self._fill_fc.append(self.color["placeholder"])
        self._fill_gc.append(self.color["regular_text"])
        self._fill_hc.append(self.color["placeholder"])
        self._fill_func.append(update_color)
        self._fill_obj.append(self.__text_obj)
        self.canvas.moveto(self.__text_obj, 16, self.h//2-10)
        self._draw_arrow(self.w-30, (self.h-12)//2)

    def bind_anim(self):
        def update_arrow():
            datas = self.__arrow_json[self._flash_t]
            y_n = 0
            for i in datas:
                x_n = 0
                for j in i:
                    self.canvas.itemconfigure(self.__arrow_list[y_n][x_n], fill=(color_tmpl.mix_color(
                        self.color["background"], self.color[self._color_bd], int((1-j/255)*1000)/1000)))
                    x_n += 1
                y_n += 1

        def anim_magictk():
            if (self._is_hover == 1 and self._flash_t < self.max_flash):
                self._flash_t += (1 if (len(self.root.anim) > 6) else 1)
                self._flash_t = min(self._flash_t, self.max_flash)
                self.hover_mode = self._flash_t/self.max_flash
                self._update_color()
            elif (self._is_hover == 0 and self._flash_t > 0):
                self._flash_t -= (1 if (len(self.root.anim) > 6) else 1)
                self._flash_t = max(self._flash_t, 0)
                self.hover_mode = self._flash_t/self.max_flash
                self._update_color()
            elif (self._is_hover == 0 and self._flash_t == 0):
                return -1
            update_arrow()

        def anim_normal(*args):
            if (self._is_hover == 1 and self._flash_t < self.max_flash):
                self._flash_t += 1
                self.hover_mode = self._flash_t/self.max_flash
                self.__update_color()
            elif (self._is_hover == 0 and self._flash_t > 0):
                self._flash_t -= 1
                self.hover_mode = self._flash_t/self.max_flash
                self.__update_color()
            self.root.after(anim_normal, 16)
            update_arrow()

        try:
            self.root.anim == 0
        except:
            self.root.after(anim_normal, 16)
        else:
            if (anim_magictk not in self.root.anim):
                self.root.anim.append(anim_magictk)
                self.__anim_obj_id = self.root.anim[-1]

    def _callback_menu(self, obj, ids):
        self.__menuobj.menu_effect[self.__last] = {}
        self.__menuobj.menu_effect[ids] = {
            "fill": self.color["primary"], "font": (obj.font[0], obj.font[1], "bold")}
        self.__last = ids
        self.canvas.itemconfigure(self.__text_obj, text=self.items[ids])
        self.canvas.moveto(self.__text_obj, 16, self.h//2-10)

    def _bind_event(self):
        def closecallback(obj):
            self._is_hover = 0

        def pressrelease_v(event):
            self.bind_anim()
            if (self._is_hover == 1):
                self.menus.close()
            else:
                self._is_hover = 1
                self.menus = submenu.Menu(
                    x=event.x_root-event.x,
                    y=event.y_root-event.y+self.h+4,
                    w=self.w,
                    h=min(8*34, len(self.items)*34+12+4),
                    root=self.root, menuobj=self.__menuobj, closecallback=closecallback
                )
            self._update_color()
        self.canvas.bind("<ButtonRelease-1>", pressrelease_v)



import json
import tkinter
from tkinter import ttk

from magictk import color_tmpl
from magictk import photoload
from magictk import fontconfig


class Button:
    color = color_tmpl.default_color
    hover_mode = 0.0
    _is_hover = 0
    _flash_t = 0
    max_flash = 4
    _anim_obj_id = -1
    text = "Button"

    def _draw_corner(self, r_x, r_y, x, y, **kwargs):
        border_info = json.loads(photoload.loadres("buttonborder"))
        y_n = 0
        for i in border_info:
            x_n = 0
            for j in i:
                if (r_x == 0):
                    px = x+x_n+1
                else:
                    px = x+4-x_n-1
                if (r_y == 0):
                    py = y+y_n+1
                else:
                    py = y+4-y_n-1
                if (j < 0):
                    lcolor = -j
                else:
                    lcolor = j
                g_color = color_tmpl.mix_color(
                    self.color["background"], self.color[self._color_bd], int((1-lcolor/255)*1000)/1000)
                if (j < 0):
                    f_color = color_tmpl.mix_color(
                        self.color[self._color_fg2], self.color[self._color_fg1], int((1-lcolor/255)*1000)/1000)
                else:
                    f_color = color_tmpl.mix_color(
                        self.color["background"], self.color[self._color_fg1], int((1-lcolor/255)*1000)/1000)
                if (j < 0):
                    h_color = color_tmpl.mix_color(
                        self.color[self._color_fg2], self.color[self._color_fg], int((1-lcolor/255)*1000)/1000)
                else:
                    h_color = color_tmpl.mix_color(
                        self.color["background"], self.color[self._color_fg], int((1-lcolor/255)*1000)/1000)

                obj = self.canvas.create_rectangle(
                    px, py, px, py, width=0, fill=g_color)

                def update_color(obj, g_color, f_color, h_color):
                    if (self._is_hover == 2):
                        self.canvas.itemconfig(
                            obj, fill=h_color)
                    else:
                        self.canvas.itemconfig(
                            obj, fill=color_tmpl.mix_color(g_color, f_color, self.hover_mode))

                self._fill_func.append(update_color)
                self._fill_gc.append(g_color)
                self._fill_fc.append(f_color)
                self._fill_hc.append(h_color)
                self._fill_obj.append(obj)
                x_n += 1
            y_n += 1

    def _update_color(self):
        n = 0
        for i in self._fill_func:
            i(self._fill_obj[n], self._fill_gc[n],
              self._fill_fc[n], self._fill_hc[n])
            n += 1

    def __init__(self, master=None, root_anim=None, w=80, h=30, text="Button", func=lambda s: print("Press"), color_list: dict = None, _set_defaultcolor=None):
        global use_font
        use_font = fontconfig.getfont()
        self._fill_obj = []
        self._fill_func = []
        self._fill_gc = []
        self._fill_fc = []
        self._fill_hc = []
        self._func = func
        if (_set_defaultcolor is None):
            self._color_bd = "border_base"
            self._color_bg = "background"
            self._color_bg1 = "background"
            self._color_fg = "primary"
            self._color_fg1 = "primary_light"
            self._color_fg2 = "primary_light2"
            self._color_fg3 = "primary_light2"
            self._color_text = None
        self.w = max(30, w)
        self.h = h
        self.text = text
        self.__master = master
        if (color_list is not None):
            self.color = color_list
        if (root_anim == None):
            self.root = master.root
        else:
            self.root = root_anim

        self.canvas = tkinter.Canvas(
            master, bg=self.color["background"], width=self.w, height=self.h, borderwidth=0, bd=0, highlightcolor=self.color["background"], highlightthickness=0)

        self._draw()
        self._update_color()
        self._bind_event()
        self.bind_anim()

    def pack(self, *args, **kwargs):
        self.canvas.pack(*args, **kwargs)

    def guid(self, *args, **kwargs):
        self.canvas.guid(*args, **kwargs)

    def place(self, *args, **kwargs):
        self.canvas.place(*args, **kwargs)

    def _draw(self, _use_self_text=None):
        self._draw_corner(0, 0, 0, 0)
        self._draw_corner(1, 0, self.w-4, 0)
        self._draw_corner(0, 1, 0, self.h-5)
        self._draw_corner(1, 1, self.w-4, self.h-5)

        def update_color(obj, g_color, f_color, h_color):
            if (self._is_hover == 2):
                self.canvas.itemconfig(
                    obj, fill=h_color)
            else:
                self.canvas.itemconfig(
                    obj, fill=color_tmpl.mix_color(g_color, f_color, self.hover_mode))
        self._fill_fc.append(self.color[self._color_fg1])
        self._fill_gc.append(self.color[self._color_bd])
        self._fill_hc.append(self.color[self._color_fg])
        self._fill_func.append(update_color)
        self._fill_obj.append(self.canvas.create_line(
            1, 5, 1, self.h-5, width=1, fill=self.color[self._color_bd]))
        self._fill_fc.append(self.color[self._color_fg1])
        self._fill_gc.append(self.color[self._color_bd])
        self._fill_hc.append(self.color[self._color_fg])
        self._fill_func.append(update_color)
        self._fill_obj.append(self.canvas.create_line(
            5, 1, self.w-4, 1, width=1, fill=self.color[self._color_fg1]))
        self._fill_fc.append(self.color[self._color_fg1])
        self._fill_gc.append(self.color[self._color_bd])
        self._fill_hc.append(self.color[self._color_fg])
        self._fill_func.append(update_color)
        self._fill_obj.append(self.canvas.create_line(
            self.w-1, 5, self.w-1, self.h-5, width=1, fill=self.color[self._color_fg1]))
        self._fill_fc.append(self.color[self._color_fg1])
        self._fill_gc.append(self.color[self._color_bd])
        self._fill_hc.append(self.color[self._color_fg])
        self._fill_func.append(update_color)
        self._fill_obj.append(self.canvas.create_line(
            5, self.h-2, self.w-4, self.h-2, width=1, fill=self.color[self._color_fg1]))

        self._fill_fc.append(self.color[self._color_fg3])
        self._fill_gc.append(self.color[self._color_bg1])
        self._fill_hc.append(self.color[self._color_fg2])
        self._fill_func.append(update_color)
        self._fill_obj.append(self.canvas.create_rectangle(
            2, 5, self.w-1, self.h-5, width=0, fill=self.color[self._color_fg2]))

        self._fill_fc.append(self.color[self._color_fg3])
        self._fill_gc.append(self.color[self._color_bg1])
        self._fill_hc.append(self.color[self._color_fg2])
        self._fill_func.append(update_color)
        self._fill_obj.append(self.canvas.create_rectangle(
            5, 2, self.w-4, self.h-2, width=0, fill=self.color[self._color_fg2]))

        if (_use_self_text is None):
            if (self._color_text is None):
                self._fill_fc.append(self.color[self._color_fg])
                self._fill_gc.append(self.color["regular_text"])
                self._fill_hc.append(self.color[self._color_fg])
            else:
                self._fill_fc.append(self._color_text)
                self._fill_gc.append(self._color_text)
                self._fill_hc.append(self._color_text)
            self._fill_func.append(update_color)
            self._fill_obj.append(
                self.canvas.create_text(int(self.w/2), int(self.h/2), text=self.text, font=(use_font, 10)))

    def bind_anim(self):
        def anim_magictk():
            if (self._is_hover == 1 and self._flash_t < self.max_flash):
                self._flash_t += (1 if (len(self.root.anim) > 6) else 1)
                self._flash_t = min(self._flash_t, self.max_flash)
                self.hover_mode = self._flash_t/self.max_flash
                self._update_color()
            elif (self._is_hover == 0 and self._flash_t > 0):
                self._flash_t -= (1 if (len(self.root.anim) > 6) else 1)
                self._flash_t = max(self._flash_t, 0)
                self.hover_mode = self._flash_t/self.max_flash
                self._update_color()
            else:
                return -1

        def anim_normal(*args):
            if (self._is_hover == 1 and self._flash_t < self.max_flash):
                self._flash_t += 1
                self.hover_mode = self._flash_t/self.max_flash
                self._update_color()
            elif (self._is_hover == 0 and self._flash_t > 0):
                self._flash_t -= 1
                self.hover_mode = self._flash_t/self.max_flash
                self._update_color()
            self.root.after(anim_normal, 16)

        try:
            self.root.anim == 0
        except:
            self.root.after(anim_normal, 16)
        else:
            if (anim_magictk not in self.root.anim):
                self.root.anim.append(anim_magictk)
                self._anim_obj_id = self.root.anim[-1]

    def _bind_event(self):
        def enter_v(*args):
            self.bind_anim()
            if (self._is_hover == 0):
                self._is_hover = 1
        self.canvas.bind("<Enter>", enter_v)

        def leave_v(*args):
            self.bind_anim()
            if (self._is_hover == 1):
                self._is_hover = 0
        self.canvas.bind("<Leave>", leave_v)

        def press_v(*args):
            self._is_hover = 2
            self._update_color()
        self.canvas.bind("<Button-1>", press_v)

        def pressrelease_v(*args):
            self.bind_anim()
            self._is_hover = 1
            self._func(self)
            self._update_color()
        self.canvas.bind("<ButtonRelease-1>", pressrelease_v)


class ButtonFill(Button):

    def _draw_corner(self, r_x, r_y, x, y, **kwargs):
        border_info = json.loads(photoload.loadres("buttonborder"))
        y_n = 0
        for i in border_info:
            x_n = 0
            for j in i:
                if (r_x == 0):
                    px = x+x_n+1
                else:
                    px = x+4-x_n-1
                if (r_y == 0):
                    py = y+y_n+1
                else:
                    py = y+4-y_n-1
                if (j < 0):
                    lcolor = -j
                else:
                    lcolor = j
                if (j < 0):
                    g_color = color_tmpl.mix_color(
                        self.color[self._color_bg], self.color[self._color_bd], int((1-lcolor/255)*1000)/1000)
                else:
                    g_color = color_tmpl.mix_color(
                        self.color["background"], self.color[self._color_bd], int((1-lcolor/255)*1000)/1000)
                if (j < 0):
                    f_color = color_tmpl.mix_color(
                        self.color[self._color_fg1], self.color[self._color_fg1], int((1-lcolor/255)*1000)/1000)
                else:
                    f_color = color_tmpl.mix_color(
                        self.color["background"], self.color[self._color_fg1], int((1-lcolor/255)*1000)/1000)
                if (j < 0):
                    h_color = color_tmpl.mix_color(
                        self.color[self._color_fg2], self.color[self._color_fg2], int((1-lcolor/255)*1000)/1000)
                else:
                    h_color = color_tmpl.mix_color(
                        self.color["background"], self.color[self._color_fg2], int((1-lcolor/255)*1000)/1000)

                obj = self.canvas.create_rectangle(
                    px, py, px, py, width=0, fill=g_color)

                def update_color(obj, g_color, f_color, h_color):
                    if (self._is_hover == 2):
                        self.canvas.itemconfig(
                            obj, fill=h_color)
                    else:
                        self.canvas.itemconfig(
                            obj, fill=color_tmpl.mix_color(g_color, f_color, self.hover_mode))

                self._fill_func.append(update_color)
                self._fill_gc.append(g_color)
                self._fill_fc.append(f_color)
                self._fill_hc.append(h_color)
                self._fill_obj.append(obj)
                x_n += 1
            y_n += 1

    def __init__(self, master=None, root_anim=None, color_type="primary", w=80, h=30, text="Button", func=lambda s: print("Press"), color_list: dict = None, _dis_color=None):
        if (_dis_color is None):
            self._color_bd = color_type
            self._color_bg = color_type
            self._color_bg1 = color_type
            self._color_fg1 = color_type+"_light3"
            self._color_fg = color_type+"_dark"
            self._color_fg3 = color_type+"_light3"
            self._color_fg2 = color_type+"_dark"
            self._color_text = "#FFFFFF"
        super().__init__(master=master, root_anim=root_anim, w=w, h=h,
                         text=text, color_list=color_list, func=func, _set_defaultcolor=True)



import time
import tkinter
import os
from tkinter import ttk
from magictk import color_tmpl
from magictk import _window_ctl
from magictk import _window_size
from magictk import workspace
from magictk import photoload
import sys

WIN_INF = -10000


class Window(ttk.Frame):
    x = 200
    y = 200
    w = 500
    h = 350
    normal_x = -1
    normal_y = 1
    top_h = 28
    __break_win = True
    fullscreen = False
    minmode = False
    color = color_tmpl.default_color
    anim = []
    disable_move = False
    min_w = 200
    min_h = 100
    FRAME_INFO = "Custom"

    def update_size(self) -> None:
        self.tk_w_without_bar.set(self.w-48*3)
        self.place(x=8, y=self.top_h+1+8, width=self.w-16, height=self.h-1-16)
        self.main_tk.geometry(
            f"{self.w}x{self.h+self.top_h}+{self.x}+{self.y}")
        self.__fake_tk.geometry(
            f"{self.w}x{self.h}+{WIN_INF}+{WIN_INF}")

    def __load_color(self) -> None:
        self.main_tk.configure(bg=self.color["background"])
        self.style.configure(
            'Test1.TFrame', background="#FF0000")
        self.style.configure(
            'Test2.TFrame', background="#00FF00")
        self.style.configure(
            'Test3.TFrame', background="#0000FF")
        self.style.configure(
            'Main.TFrame', background=self.color["background"])
        self.style.configure(
            'Root.TFrame', background=self.color["background"])
        self.style.configure(
            'Splitline.TFrame', background=self.color["border_light"])
        self.style.configure(
            'BarExitButton.TFrame', background=self.color["background"])
        self.style.configure(
            'LabBarZoomButton.TLabel', background=self.color["background"])
        self.style.configure(
            'LabBarIcon.TLabel', background=self.color["background"])
        self.style.configure(
            'LabTitle.TLabel', background=self.color["background"], fg=self.color["primary_text"])
        self.style.configure(
            'BarIconicButton.TFrame', background=self.color["background"])
        self.style.configure(
            'Size.TFrame', background=self.color["background"])
        self.style.configure(
            'LabelTitle.TFrame', background=self.color["background"])

    def __init_weights(self, root: tkinter.Frame) -> None:
        top_bar = ttk.Frame(root, style='Root.TFrame', height=self.top_h)
        startmove_x = 0
        startmove_y = 0
        title_frame = ttk.Frame(
            top_bar, style="LabelTitle.TFrame", height=self.top_h)

        def start_move(event: tkinter.Event):
            if (self.disable_move):
                return
            nonlocal startmove_x, startmove_y
            startmove_x = event.x_root-self.main_tk.winfo_x()
            startmove_y = event.y_root-self.main_tk.winfo_y()
            if (self.fullscreen):
                self.zoom()
                self.x = event.x_root-(event.x_root-self.main_tk.winfo_x())
                self.y = event.y_root-(event.y_root-self.main_tk.winfo_y())
                startmove_x = int(self.w/2)
                startmove_y = int(self.top_h/2)
                self.update_size()
            title_frame.configure(cursor="arrow")

        def move_win(event: tkinter.Event):
            if (self.disable_move):
                return
            nonlocal startmove_x, startmove_y
            if (self.fullscreen):
                return
            self.x = event.x_root-startmove_x
            self.y = event.y_root-startmove_y
            if (event.y_root < 5):
                title_frame.configure(cursor="dotbox")
            else:
                title_frame.configure(cursor="fleur")
            self.update_size()

        def special_move(event: tkinter.Event):
            if (event.y_root < 5):
                self.y = 50
                self.zoom()
            title_frame.configure(cursor="arrow")
        top_bar.place(x=0, y=0, relwidth=1)
        ttk.Frame(root, style='Splitline.TFrame',
                  height=1).place(x=0, y=self.top_h, relwidth=1)

        icons = ttk.Label(top_bar, image=photoload.loadimg(
            "icon"), width=2, compound="center", justify="center", style="LabBarIcon.TLabel")
        icons.bind("<ButtonRelease-2>", lambda event=None: self.quit())
        icons.pack(anchor="center", side='left', padx=8, pady=4)

        title_frame.pack(anchor="center", side='left',
                         fill=tkinter.X, expand=True)
        control_frame = ttk.Frame(
            top_bar, style="Root.TFrame", height=self.top_h, width=48*3)
        control_frame.pack(anchor="e", side='right')
        title_frame.bind("<Button-1>", start_move)
        title_frame.bind("<B1-Motion>", move_win)
        title_frame.bind("<ButtonRelease-1>", special_move)

        _window_ctl.exit_btn_set(self, root)
        _window_ctl.zoom_btn_set(self, root)
        _window_ctl.iconic_btn_set(self, root)
        _window_size.placeall(self, root)

        titles = ttk.Label(title_frame, text=self.title,
                           style="LabTitle.TLabel")
        titles.pack()
        titles.bind("<Button-1>", start_move)
        titles.bind("<B1-Motion>", move_win)
        titles.bind("<ButtonRelease-1>", special_move)

    def __init__(self, w=500, h=350, title="MagicTk", color_list: dict = None) -> None:
        self.root = self
        self.title = title
        self.w = w
        self.h = h
        if (color_list is not None):
            self.color = color_list
        self.main_tk = tkinter.Tk()
        self.main_tk.overrideredirect(True)
        self.__fake_tk = tkinter.Tk()
        self.style = ttk.Style()
        self.__load_color()
        self.tk_w_without_bar = tkinter.IntVar()
        self.tk_w_without_bar.set(self.w-48*3)

        ttk.Widget.__init__(self, self.main_tk, "ttk::frame", {
                            "style": "Main.TFrame"})
        self.place(x=8, y=self.top_h+1+8, width=self.w-16, height=self.h-1-16)

        self.update_size()
        self.__fake_tk.title(self.title)
        self.main_tk.title(self.title)
        self.__fake_tk.protocol("WM_DELETE_WINDOW", self.quit)
        self.__fake_tk.bind('<FocusIn>', self.__top_window)
        self.__fake_tk.resizable(0, 0)
        self.__init_weights(self.main_tk)
        try:
            self.main_tk.iconbitmap(
                os.path.dirname(__file__)+os.sep+"icon.ico")
            self.__fake_tk.iconbitmap(
                os.path.dirname(__file__)+os.sep+"icon.ico")
        except:
            # What's fuck in Linux
            self.zoom()
            self.main_tk.update()
            self.__fake_tk.update()
            self.zoom()
            self.main_tk.update()
            self.__fake_tk.update()

    def mainloop(self) -> None:
        t_start = time.time()
        while (self.__break_win):
            delta_t = time.time()-t_start
            if (delta_t > 0.02):  # flash animation
                t_start = time.time()
                n = 0
                for i in self.anim:
                    if (i is not None):
                        retn = i()
                        if (retn == -1):
                            self.anim[n] = None
                    n += 1
                self.anim = [i for i in self.anim if i is not None]
            else:
                pass
            if (self.__fake_tk.state() == "iconic"):
                if (self.normal_x == -1):
                    self.normal_x = self.x
                    self.normal_y = self.y
                    self.x = WIN_INF
                    self.y = WIN_INF
                    self.minmode = True
                    self.fullscreen = False
                    self.update_size()
            else:
                if (self.normal_x != -1):
                    self.x = self.normal_x
                    self.y = self.normal_y
                    self.normal_x = -1
                    self.minmode = False
                    self.update_size()
            self.main_tk.update()
            self.__fake_tk.update()
        self.main_tk.destroy()
        self.__fake_tk.destroy()
        self.main_tk.mainloop()
        self.__break_win = None

    def quit(self) -> None:
        self.__break_win = False

    def iconify(self) -> None:
        if (self.fullscreen == True):
            self.zoom()
        self.__fake_tk.iconify()

    def __top_window(self, event=None) -> None:
        self.main_tk.state("normal")
        self.main_tk.focus_set()
        self.main_tk.attributes('-topmost', 'true')
        self.main_tk.update()
        self.main_tk.attributes('-topmost', 'false')

    def zoom(self) -> None:
        if (self.fullscreen == False):
            self.normal_w = self.w
            self.normal_h = self.h
            self.old_x = self.x
            self.old_y = self.y
            nx = 0
            ny = 0
            nw, nh = self.__fake_tk.maxsize()
            try:
                wsp = workspace.get_workspace_size()
                nx = wsp[0]
                ny = wsp[1]
                nw = wsp[2]
                nh = wsp[3]-self.top_h
            except:
                pass
            self.normal_x = -1
            self.x = nx
            self.y = ny
            self.main_tk.geometry(
                f"+{self.x}+{self.y}")
            self.main_tk.update()
            self.w = nw
            self.h = nh
            self.update_size()
            self.fullscreen = True
            self.minmode = False
            self.__top_window()
        else:
            self.normal_x = -1
            self.x = self.old_x
            self.y = self.old_y
            self.main_tk.geometry(
                f"+{self.x}+{self.y}")
            self.main_tk.update()
            self.w = self.normal_w
            self.h = self.normal_h
            self.fullscreen = False
            self.minmode = False
            self.update_size()
            self.__top_window()
        self.update_icon()



import tkinter
from tkinter import ttk


def place_move_up(self, root):
    global change_bar_move_up, up_move_start, up_move
    move_up = ttk.Frame(
        root, style="Size.TFrame", height=6, width=self.tk_w_without_bar.get(), cursor="top_side")

    def change_bar_move_up(*args, **kwargs):
        move_up.configure(width=self.tk_w_without_bar.get())
    self.tk_w_without_bar.trace_add('write', change_bar_move_up)
    move_up.place(x=0, y=0)
    upstart_h = self.h
    upstart_y = self.y

    def up_move_start(event: tkinter.Event):
        # self.disable_move = True
        # print(1)
        nonlocal upstart_h, upstart_y
        upstart_h = self.h-self.top_h
        upstart_y = event.y_root

    def up_move(event: tkinter.Event):
        nonlocal upstart_h, upstart_y
        if (self.fullscreen):
            return
        now_y = event.y_root
        delta_y = upstart_y-now_y
        now_h = upstart_h+delta_y
        if (now_h < self.min_h):
            return
        self.normal_x = -1
        self.y = upstart_y-delta_y
        self.h = now_h+self.top_h
        self.update_size()
        self.main_tk.update()
    move_up.bind("<Button-1>", up_move_start)
    move_up.bind("<B1-Motion>", up_move)


def place_move_up_bar(self, root):
    move_up_bar = ttk.Frame(
        root, style="Size.TFrame", height=1, width=48*3, cursor="top_side")

    def change_bar_move_up(*args, **kwargs):
        move_up_bar.place(x=self.tk_w_without_bar.get(), y=0)
    self.tk_w_without_bar.trace_add('write', change_bar_move_up)

    self.tk_w_without_bar.trace_add('write', change_bar_move_up)
    move_up_bar.place(y=0, x=self.tk_w_without_bar.get())
    move_up_bar.bind("<Button-1>", up_move_start)
    move_up_bar.bind("<B1-Motion>", up_move)


def place_move_down(self, root):
    global down_move_start, down_move
    move_down = ttk.Frame(
        root, style="Size.TFrame", height=4, width=self.tk_w_without_bar.get(), cursor="bottom_side")

    def change_bar_move_up(*args, **kwargs):
        move_down.place(x=0, y=self.top_h+self.h-4, relwidth=1)
    self.tk_w_without_bar.trace_add('write', change_bar_move_up)
    move_down.place(x=0, y=self.top_h+self.h-4, relwidth=1)
    downstart_h = self.h
    downstart_y = self.y

    def down_move_start(event: tkinter.Event):
        nonlocal downstart_h, downstart_y
        downstart_h = self.h-self.top_h
        downstart_y = event.y_root

    def down_move(event: tkinter.Event):
        nonlocal downstart_h, downstart_y
        if (self.fullscreen):
            return
        now_y = event.y_root
        delta_y = downstart_y-now_y
        now_h = downstart_h-delta_y
        if (now_h < self.min_h):
            return
        self.normal_x = -1
        self.h = now_h+self.top_h
        self.update_size()
        self.main_tk.update()
    move_down.bind("<Button-1>", down_move_start)
    move_down.bind("<B1-Motion>", down_move)


def place_move_left(self, root):
    global left_move_start, left_move
    move_left = ttk.Frame(
        root, style="Size.TFrame", width=6, height=self.top_h, cursor="left_side")

    move_left.place(x=0, y=0)
    leftstart_w = self.w
    leftstart_x = self.x

    def left_move_start(event: tkinter.Event):
        # self.disable_move = True
        # print(1)
        nonlocal leftstart_w, leftstart_x
        leftstart_w = self.w
        leftstart_x = event.x_root

    def left_move(event: tkinter.Event):
        nonlocal leftstart_w, leftstart_x
        if (self.fullscreen):
            return
        now_x = event.x_root
        delta_x = leftstart_x-now_x
        now_w = leftstart_w+delta_x
        if (now_w < self.min_w):
            return
        self.normal_x = -1
        self.x = leftstart_x-delta_x
        self.w = now_w
        self.update_size()
        self.main_tk.update()
    move_left.bind("<Button-1>", left_move_start)
    move_left.bind("<B1-Motion>", left_move)


def place_move_left_bar(self, root):
    move_left = ttk.Frame(
        root, style="Size.TFrame", width=4, height=self.h, cursor="left_side")

    def change_bar_move_up(*args, **kwargs):
        move_left.configure(height=self.h)
    self.tk_w_without_bar.trace_add('write', change_bar_move_up)
    move_left.place(x=0, y=self.top_h)

    move_left.bind("<Button-1>", left_move_start)
    move_left.bind("<B1-Motion>", left_move)


def place_move_right(self, root):
    global right_move, right_move_start
    move_right = ttk.Frame(
        root, style="Size.TFrame", width=1, height=self.top_h, cursor="right_side")

    def change_bar_move(*args, **kwargs):
        move_right.place(x=self.w-1, y=0)
    self.tk_w_without_bar.trace_add('write', change_bar_move)
    move_right.place(x=self.w-1, y=0)
    rightstart_w = self.w
    rightstart_x = self.x

    def right_move_start(event: tkinter.Event):
        # self.disable_move = True
        # print(1)
        nonlocal rightstart_w, rightstart_x
        rightstart_w = self.w
        rightstart_x = event.x_root

    def right_move(event: tkinter.Event):
        nonlocal rightstart_w, rightstart_x
        if (self.fullscreen):
            return
        now_x = event.x_root
        delta_x = rightstart_x-now_x
        now_w = rightstart_w-delta_x
        if (now_w < self.min_w):
            return
        self.normal_x = -1
        self.w = now_w
        self.update_size()
        self.main_tk.update()
    move_right.bind("<Button-1>", right_move_start)
    move_right.bind("<B1-Motion>", right_move)


def place_move_right_bar(self, root):
    move_right = ttk.Frame(
        root, style="Size.TFrame", width=4, height=self.h, cursor="right_side")

    def change_bar_move_right(*args, **kwargs):
        move_right.place(x=self.w-4, y=self.top_h)
        move_right.configure(height=self.h)
    self.tk_w_without_bar.trace_add('write', change_bar_move_right)
    move_right.place(x=self.w-4, y=self.top_h)
    move_right.bind("<Button-1>", right_move_start)
    move_right.bind("<B1-Motion>", right_move)


def place_move_LT_bar(self, root):
    move_LT = ttk.Frame(
        root, style="Size.TFrame", width=8, height=8, cursor="top_left_corner")

    move_LT.place(x=0, y=0)

    leftstart_w = self.w
    leftstart_x = self.x
    upstart_h = self.h
    upstart_y = self.y

    def LT_move_start(event=None):
        nonlocal leftstart_w, leftstart_x
        leftstart_w = self.w
        leftstart_x = event.x_root
        nonlocal upstart_h, upstart_y
        upstart_h = self.h-self.top_h
        upstart_y = event.y_root

    def LT_move(event=None):
        while 1:
            nonlocal leftstart_w, leftstart_x
            if (self.fullscreen):
                break
            now_x = event.x_root
            delta_x = leftstart_x-now_x
            now_w = leftstart_w+delta_x
            if (now_w < self.min_w):
                break
            self.normal_x = -1
            self.x = leftstart_x-delta_x
            self.w = now_w
            break
        while 1:
            nonlocal upstart_h, upstart_y
            if (self.fullscreen):
                break
            now_y = event.y_root
            delta_y = upstart_y-now_y
            now_h = upstart_h+delta_y
            if (now_h < self.min_h):
                break
            self.normal_x = -1
            self.y = upstart_y-delta_y
            self.h = now_h+self.top_h
            break
        self.update_size()
        self.main_tk.update()

    move_LT.bind("<Button-1>", LT_move_start)
    move_LT.bind("<B1-Motion>", LT_move)


def place_move_RT_bar(self, root):
    move_RT = ttk.Frame(
        root, style="Size.TFrame", width=2, height=2, cursor="top_right_corner")

    def change_bar_move(*args, **kwargs):
        move_RT.place(x=self.w-2, y=0)
    self.tk_w_without_bar.trace_add('write', change_bar_move)
    move_RT.place(x=self.w-2, y=0)

    upstart_h = self.h
    upstart_y = self.y
    rightstart_w = self.w
    rightstart_x = self.x

    def RT_move_start(event=None):
        nonlocal upstart_h, upstart_y
        upstart_h = self.h-self.top_h
        upstart_y = event.y_root
        nonlocal rightstart_w, rightstart_x
        rightstart_w = self.w
        rightstart_x = event.x_root

    def RT_move(event=None):
        while 1:
            nonlocal upstart_h, upstart_y
            if (self.fullscreen):
                break
            now_y = event.y_root
            delta_y = upstart_y-now_y
            now_h = upstart_h+delta_y
            if (now_h < self.min_h):
                break
            self.normal_x = -1
            self.y = upstart_y-delta_y
            self.h = now_h+self.top_h
            break
        while 1:
            nonlocal rightstart_w, rightstart_x
            if (self.fullscreen):
                break
            now_x = event.x_root
            delta_x = rightstart_x-now_x
            now_w = rightstart_w-delta_x
            if (now_w < self.min_w):
                break
            self.normal_x = -1
            self.w = now_w
            break
        self.update_size()
        self.main_tk.update()

    move_RT.bind("<Button-1>", RT_move_start)
    move_RT.bind("<B1-Motion>", RT_move)


def place_move_RB_bar(self, root):
    move_RB = ttk.Frame(
        root, style="Size.TFrame", width=6, height=6, cursor="bottom_right_corner")

    def change_bar_move(*args, **kwargs):
        move_RB.place(x=self.w-6, y=self.h-6+self.top_h)
    self.tk_w_without_bar.trace_add('write', change_bar_move)
    move_RB.place(x=self.w-6, y=self.h-6+self.top_h)

    rightstart_x = self.x
    rightstart_w = self.w
    downstart_h = self.h
    downstart_y = self.y

    def RB_move_start(event=None):
        nonlocal rightstart_w, rightstart_x
        rightstart_w = self.w
        rightstart_x = event.x_root
        nonlocal downstart_h, downstart_y
        downstart_h = self.h-self.top_h
        downstart_y = event.y_root

    def RB_move(event=None):
        while 1:
            nonlocal rightstart_w, rightstart_x
            if (self.fullscreen):
                break
            now_x = event.x_root
            delta_x = rightstart_x-now_x
            now_w = rightstart_w-delta_x
            if (now_w < self.min_w):
                break
            self.normal_x = -1
            self.w = now_w
            break
        while 1:
            nonlocal downstart_h, downstart_y
            if (self.fullscreen):
                break
            now_y = event.y_root
            delta_y = downstart_y-now_y
            now_h = downstart_h-delta_y
            if (now_h < self.min_h):
                break
            self.normal_x = -1
            self.h = now_h+self.top_h
            break
        self.update_size()
        self.main_tk.update()

    move_RB.bind("<Button-1>", RB_move_start)
    move_RB.bind("<B1-Motion>", RB_move)


def place_move_LB_bar(self, root):
    move_LB = ttk.Frame(
        root, style="Size.TFrame", width=4, height=4, cursor="bottom_left_corner")

    def change_bar_move(*args, **kwargs):
        move_LB.place(x=0, y=self.h-4+self.top_h)
    self.tk_w_without_bar.trace_add('write', change_bar_move)
    move_LB.place(x=0, y=self.h-4+self.top_h)

    leftstart_w = self.w
    leftstart_x = self.x
    downstart_h = self.h
    downstart_y = self.y

    def LB_move_start(event=None):
        nonlocal leftstart_w, leftstart_x
        leftstart_w = self.w
        leftstart_x = event.x_root
        nonlocal downstart_h, downstart_y
        downstart_h = self.h-self.top_h
        downstart_y = event.y_root

    def LB_move(event=None):
        while 1:
            nonlocal leftstart_w, leftstart_x
            if (self.fullscreen):
                break
            now_x = event.x_root
            delta_x = leftstart_x-now_x
            now_w = leftstart_w+delta_x
            if (now_w < self.min_w):
                break
            self.normal_x = -1
            self.x = leftstart_x-delta_x
            self.w = now_w
            break
        while 1:
            nonlocal downstart_h, downstart_y
            if (self.fullscreen):
                break
            now_y = event.y_root
            delta_y = downstart_y-now_y
            now_h = downstart_h-delta_y
            if (now_h < self.min_h):
                break
            self.normal_x = -1
            self.h = now_h+self.top_h
            break
        self.update_size()
        self.main_tk.update()

    move_LB.bind("<Button-1>", LB_move_start)
    move_LB.bind("<B1-Motion>", LB_move)


def placeall(self, root):
    place_move_up(self, root)
    place_move_up_bar(self, root)
    place_move_down(self, root)
    place_move_left(self, root)
    place_move_left_bar(self, root)
    place_move_right(self, root)
    place_move_right_bar(self, root)
    place_move_LT_bar(self, root)
    place_move_RT_bar(self, root)
    place_move_LB_bar(self, root)
    place_move_RB_bar(self, root)



import sys
import os
from setuptools import setup

####### config #######
VERSION = "0.0.2"
######################

set_v = VERSION
if (os.path.exists("runid.conf")):
    with open("runid.conf", "r") as file:
        runid = file.read()
    set_v = set_v+"."+runid
else:
    set_v = set_v+"."+"1000"

print("BUILD: version="+set_v)

print("----- setup -----")
setup(
    name="magictk",
    version=set_v,
    packages=("magictk",),
    package_dir={
        "magictk": "./magictk",
    },
    python_requires='>=3.8',
    include_package_data=True,
    install_requires=[],
    author='cxykevin|git.hmtsai.cn',
    author_email='cxykevin@yeah.net',
    description='A tkinter weights looks like element-plus',
    long_description='',
    url='http://git.hmtsai.cn/cxykevin/magictk.git',
    license='GPLv2',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3.12',
    ]
)



import tkinter
from tkinter import ttk
from magictk import color_tmpl


class Frame(tkinter.Frame):
    color = color_tmpl.default_color

    def __init__(self, master, color=None, *args, **kwargs):
        self.root = master.root
        kwargs["bg"] = self.color["background"]
        super().__init__(master, *args, **kwargs)
        self.configure()



import json
import tkinter
from tkinter import ttk
from magictk import color_tmpl
from magictk import photoload


class ProgressBar:
    color = color_tmpl.default_color
    __fill_obj = {}
    progress = 0.0
    __progress_pixel = 0
    __move_progress_pixel = 0
    __flash_t = 0
    max_flash = 4
    __anim_obj_id = -1

    def __draw_corner(self, r_x, r_y, x, y, colors="#000000", rid="", bgcolor=None, **kwargs):
        if bgcolor is None or self.progress == 1:
            bgcolor = self.color["border_light"]
        self.__fill_obj[rid] = []
        self.__fill_obj[rid+"_pos"] = []
        border_info = json.loads(photoload.loadres("progressborder"))
        y_n = 0
        for i in border_info:
            x_n = 0
            for j in i:
                if (r_x == 0):
                    px = x+x_n+1
                else:
                    px = x+3-x_n-1
                if (r_y == 0):
                    py = y+y_n+1
                else:
                    py = y+6-y_n-1
                if (j < 0):
                    lcolor = -j
                else:
                    lcolor = j
                # if(lcolor==255):
                #     continue
                g_color = color_tmpl.mix_color(
                    bgcolor, colors, int((1-lcolor/255)*1000)/1000)

                obj = self.canvas.create_rectangle(
                    px, py, px, py, width=0, fill=g_color)

                self.__fill_obj[rid].append(obj)
                self.__fill_obj[rid+"_pos"].append([x_n, y_n])

                x_n += 1
            y_n += 1

    def __init__(self, master=None, root_anim=None, w=200, h=8, colors="primary", color_list: dict = None):
        self.w = max(200, w)
        self.h = 8
        self.__master = master
        self.colors = colors
        if (color_list is not None):
            self.color = color_list
        if (root_anim == None):
            self.__root = master.root
        else:
            self.__root = root_anim

        self.canvas = tkinter.Canvas(
            master, bg=self.color["background"], width=self.w, height=self.h, borderwidth=0, bd=0, highlightcolor=self.color["background"], highlightthickness=0)

        self.__draw()
        # self.__bind_event()
        self.bind_anim()

    def pack(self, *args, **kwargs):
        self.canvas.pack(*args, **kwargs)

    def guid(self, *args, **kwargs):
        self.canvas.guid(*args, **kwargs)

    def place(self, *args, **kwargs):
        self.canvas.place(*args, **kwargs)

    def __draw(self):
        self.__fill_obj["bgbar"] = [
            self.canvas.create_rectangle(4, 1, self.w-5, self.h-1, width=0, fill=self.color["border_light"])]
        self.__draw_corner(1, 0, self.w-5, 0,
                           self.color["border_light"], "rightside", bgcolor=self.color["background"])
        self.__fill_obj["fgbar"] = [
            self.canvas.create_rectangle(4, 1, self.__progress_pixel+4, self.h-1, width=0, fill=self.color[self.colors])]
        self.__draw_corner(
            0, 0, 1, 0, self.color[self.colors], "leftside", bgcolor=self.color["background"])
        self.__draw_corner(1, 0, self.__progress_pixel+3, 0,
                           self.color[self.colors], "rightprog")

    def __update_pixel(self):
        self.__move_progress_pixel = max(4, int((self.w-6)*self.progress))

    def add_progress(self, n):
        self.progress = max(min(self.progress+n, 1.0), 0.0)
        self.__update_pixel()

    def set_progress(self, n):
        self.progress = max(min(n, 1.0), 0.0)
        self.__update_pixel()

    def __update_prog(self):
        for i in range(len(self.__fill_obj["rightprog"])):
            self.canvas.coords(
                self.__fill_obj["rightprog"][i], self.__progress_pixel+3-int(self.__fill_obj["rightprog_pos"][i][0]), self.__fill_obj["rightprog_pos"][i][1]+1, self.__progress_pixel+3-int(self.__fill_obj["rightprog_pos"][i][0])+1, self.__fill_obj["rightprog_pos"][i][1]+1)
        self.canvas.coords(
            self.__fill_obj["fgbar"][0], 4, 1, self.__progress_pixel+4, self.h-1)

    def bind_anim(self):
        def anim_magictk():
            if (int(self.__progress_pixel) < int(self.__move_progress_pixel)):
                if (self.__move_progress_pixel-self.__progress_pixel > 8):
                    self.__progress_pixel += 8
                else:
                    add_n = int(
                        (self.__progress_pixel-self.__move_progress_pixel)/2)
                    if (add_n <= 2):
                        self.__progress_pixel = self.__move_progress_pixel
                    else:
                        self.__progress_pixel += add_n
                self.__update_prog()
            elif (int(self.__progress_pixel) > int(self.__move_progress_pixel)):
                if (self.__progress_pixel-self.__move_progress_pixel > 8):
                    self.__progress_pixel -= 8
                else:
                    add_n = int(
                        (self.__progress_pixel-self.__move_progress_pixel)/2)
                    if (add_n <= 2):
                        self.__progress_pixel = self.__move_progress_pixel
                    else:
                        self.__progress_pixel -= add_n
                self.__update_prog()
            # else:
            #     self.__progress_pixel = self.__move_progress_pixel
            #     self.__update_prog()

        def anim_normal(*args):
            self.__root.after(anim_normal, 16)

        try:
            self.__root.anim == 0
        except:
            self.__root.after(anim_normal, 16)
        else:
            if (anim_magictk not in self.__root.anim):
                self.__root.anim.append(anim_magictk)
                self.__anim_obj_id = self.__root.anim[-1]
